use std::{error::Error, sync::Arc};

pub mod trivial;

use clickhouse_pool::{
    config::{ClickhouseConfig, DatalakeConfig, RetryConfig},
    pool_manager::PoolManager,
    traits::Model,
};
use tracing::{error, info, instrument};
use trivial::Trivial;

use crate::config::PersistenceConfig;

pub fn create_pool_manager(db_config: PersistenceConfig) -> Result<Arc<DatalakeConfig>, String> {
    let mut config = ClickhouseConfig::new(
        db_config.host.clone(),
        db_config.port,
        db_config.database.clone(),
        db_config.user.clone(),
        db_config.password.clone(),
        10,
        30,
        5,
    );

    config.proto = "http".to_string();

    let retry_config = RetryConfig::default();

    let datalake_config = Arc::new(DatalakeConfig::new(config, retry_config));

    Ok(datalake_config)
}

#[instrument(skip(datalake_config))]
pub async fn create_manager_and_init(
    datalake_config: Arc<DatalakeConfig>,
) -> Result<Arc<PoolManager>, Box<dyn Error>> {
    let manager = PoolManager::new(datalake_config, None).await;

    let _ = match manager
        .execute_with_retry(Trivial::create_table_sql())
        .await
    {
        Ok(_) => {
            info!("Table {} created successfully", Trivial::table_name());
        }
        Err(e) => {
            error!("Failed to create table {} : {}", Trivial::table_name(), e);
            return Err(Box::new(e));
        }
    };

    Ok(Arc::new(manager))
}
